package com.example.course.utils;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * Token工具类
 */
@Slf4j
public class TokenUtils {

    // 密钥
    private static final String SECRET_KEY = "im_wechat_secret_key_2023";

    // token有效期（7天）
    private static final long EXPIRATION_TIME = 7 * 24 * 60 * 60 * 1000;

    // token前缀
    private static final String TOKEN_PREFIX = "Bearer ";

    // header中的token键名
    private static final String HEADER_STRING = "Authorization";

    /**
     * 生成token
     * @param userId 用户ID
     * @return token字符串
     */
    public static String generateToken(Long userId) {
        Map<String, Object> claims = new HashMap<>();
        claims.put("userId", userId);

        return Jwts.builder()
                .setClaims(claims)
                .setSubject(userId.toString())
                .setIssuedAt(new Date())
                .setExpiration(new Date(System.currentTimeMillis() + EXPIRATION_TIME))
                .signWith(SignatureAlgorithm.HS512, SECRET_KEY)
                .compact();
    }

    /**
     * 从token中获取用户ID
     * @param token token字符串
     * @return 用户ID
     */
    public static Long getUserIdFromToken(String token) {
        try {
            Claims claims = Jwts.parser()
                    .setSigningKey(SECRET_KEY)
                    .parseClaimsJws(token.replace(TOKEN_PREFIX, ""))
                    .getBody();

            return Long.parseLong(claims.getSubject());
        } catch (Exception e) {
            log.error("从token中获取用户ID失败", e);
            return null;
        }
    }

    /**
     * 验证token是否有效
     * @param token token字符串
     * @return 是否有效
     */
    public static boolean validateToken(String token) {
        try {
            Jwts.parser().setSigningKey(SECRET_KEY).parseClaimsJws(token.replace(TOKEN_PREFIX, ""));
            return true;
        } catch (Exception e) {
            log.error("验证token失败", e);
            return false;
        }
    }

    /**
     * 从请求上下文中获取token
     * @return token字符串
     */
    public static String getTokenFromContext() {
        try {
            ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
            if (attributes != null) {
                HttpServletRequest request = attributes.getRequest();
                String token = request.getHeader(HEADER_STRING);
                if (token != null && token.startsWith(TOKEN_PREFIX)) {
                    return token.replace(TOKEN_PREFIX, "");
                }
            }
        } catch (Exception e) {
            log.error("从请求上下文中获取token失败", e);
        }
        return null;
    }

    /**
     * 从请求上下文中获取用户ID
     * @return 用户ID
     */
    public static Long getUserIdFromContext() {
        String token = getTokenFromContext();
        if (token != null) {
            return getUserIdFromToken(token);
        }
        return null;
    }
}
